package jfilesearch;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

/**
 * Title:        Java File Search and Download Engine
 * Description:  A simple java servlets and JSPs implementation of a search engine.
 * Copyright:    Copyright (c) 2002
 * Company:      FMI
 * @author Kaloian Manassiev
 * @version 1.0
 */

public class CFileDownloadServlet extends HttpServlet
{
  // Class' static constants
  public static final String DOWNLOAD_FILE_PARAMETER = "DownlF";

  // The content types table
  private static final String[][] CONTENT_TYPES = { { ".au", "audio/basic" },
                                                    { ".avi", "video/x-msvideo" },
                                                    { ".bmp", "image/bmp" },
                                                    { ".css", "text/css" },
                                                    { ".doc", "application/msword" },
                                                    { ".dll", "application/x-msdownload" },
                                                    { ".exe", "application/x-msdownload" },
                                                    { ".gif", "image/gif" },
                                                    { ".htm", "text/html" },
                                                    { ".html", "text/html" },
                                                    { ".jpe", "image/jpeg" },
                                                    { ".jpeg", "image/jpeg" },
                                                    { ".jpg", "image/jpeg" },
                                                    { ".js", "application/x-javascript" },
                                                    { ".mid", "audio/midi" },
                                                    { ".midi", "audio/midi" },
                                                    { ".mov", "video/quicktime" },
                                                    { ".mpeg", "video/mpeg" },
                                                    { ".mpg", "video/mpeg" },
                                                    { ".mp2", "audio/mpeg" },
                                                    { ".mp3", "audio/mpeg" },
                                                    { ".pdf", "application/pdf" },
                                                    { ".png", "image/png" },
                                                    { ".ps", "application/postscript" },
                                                    { ".ra", "audio/x-realaudio" },
                                                    { ".tar", "application/x-tar" },
                                                    { ".rtf", "application/rtf" },
                                                    { ".tif", "image/tiff" },
                                                    { ".tiff", "image/tif" },
                                                    { ".txt", "text/plain" },
                                                    { ".wav", "audio/x-wav" },
                                                    { ".xls", "application/vnd.ms-excel" },
                                                    { ".xml", "text/xml" },
                                                    { ".zip", "application/x-zip-compressed" } };

  // Generic content
  private static final String GENERIC_CONTENT = "application/octet-stream";

  //////////////////////////// PRIVATE BUSINESS METHODS /////////////////////////

  /**
   *  Retrieves the content type of a specified file
   */
  private String GetFileContentType( File f )
  {
    // The file name itself
    String str = f.getName();

    // The content type holder
    String strContentType = GENERIC_CONTENT;

    int iDot = str.lastIndexOf( "." );

    if ( iDot > 0 )
    {
      String strExtension = str.substring( iDot );

      for ( int i = 0; i < CONTENT_TYPES.length; i++ )
        if ( strExtension.equalsIgnoreCase( CONTENT_TYPES[ i ][ 0 ] ) )
        {
          strContentType = CONTENT_TYPES[ i ][ 1 ];
          break;
        }
    }

    return strContentType;
  }

  /**
   *  Sent the file contents.
   *
   *  Parameters:
   *  f  - the file to be sent
   *  out - the output stream
   */
  private void SendFile( File f, OutputStream os )
  {
    if ( null == f )
      return;

    // Open a stream to the file
    try
    {
      // The file reading input stream
      FileInputStream is = new FileInputStream( f );

      final int BUFFER_SIZE = 1024;

      // The input buffer
      byte[] buf = new byte[ BUFFER_SIZE ];

      int cRead = 0;

      while ( ( cRead = is.read( buf ) ) > 0 )
      {
        os.write( buf, 0, cRead );
      }

      // Flush the output stream
      os.flush();

      // Close the file sent
      is.close();
    }
    catch ( IOException e )
    {
      // TODO: Maybe delegate to the error page
      e.printStackTrace();
    }
  }

  /**
   * Initialization of the global variables
   * */
  public void init( ServletConfig config ) throws ServletException
  {
    super.init( config );
  }

  /**
   * Process the HTTP Get request
   * */
  public void doGet( HttpServletRequest request, HttpServletResponse response )
    throws ServletException, IOException
  {
    // The file name to be downloaded
    String  strDownloadFile = null;

    try
    {
      // Read request parameters
      strDownloadFile = request.getParameter( DOWNLOAD_FILE_PARAMETER );
    }
    catch( Exception e )
    {
      e.printStackTrace();

      // Clear the found items list
      strDownloadFile = null;
    }

    // Open a connection to that file
    File oFile = new File( strDownloadFile );

    // Set the content type
    response.setContentType( GetFileContentType( oFile ) );
    response.setContentLength( (int)oFile.length() );

    // Obatin the output stream writer
    ServletOutputStream os = response.getOutputStream();

    // Send the file
    SendFile( oFile, os );

    // Release the file handle
    oFile = null;
  }

  /**
   * Process the HTTP Post request
   * */
  public void doPost( HttpServletRequest request, HttpServletResponse response )
    throws ServletException, IOException
  {
    // Delegate to the Get method handler
    doGet( request, response );
  }

  /**
   * Clean up allocated resources
   * */
  public void destroy() {
  }
}
