package jfilesearch;

/**
 * Title:        Java File Search and Download Engine
 * Description:  A simple java servlets and JSPs implementation of a search engine.
 * Copyright:    Copyright (c) 2002
 * Company:      FMI
 * @author Kaloian Manassiev
 * @version 1.0
 */

// Imports
import java.io.File;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.util.LinkedList;
import java.util.Iterator;

// Exceptions
import java.io.FileNotFoundException;
import java.io.IOException;

public class CFileSearch
{
  // Class' static constants
  private static final int  BUFFER_LENGTH = 1024;

  // Private class data
  private String  m_strSearchRoot;

  /**
   * Contrctruction / Initialization
   *
   * Parameters:
   * 1) strRoot - the search root
   */
  public CFileSearch( String strRoot )
  {
    m_strSearchRoot = strRoot;
  }

  /////////////////////// EXPOSED METHODS //////////////////////
  public String[] DoSearch( String strNamePattern, String strContentPattern )
  {
    return SearchDirectory( m_strSearchRoot, strNamePattern, strContentPattern );
  }

  /////////////////////// PRIVATE BUSINESS METHODS //////////////////////

  /**
   * Search a specifiead directory and all its subfolders
   */
  private String[] SearchDirectory( String strPath, String strNamePattern, String strContentPattern )
  {
    LinkedList lst = new LinkedList();

    File oFile = new File( strPath );

    if ( oFile.isDirectory() )
    {
      File[] lstFiles = oFile.listFiles();

      // Iterate the sub-entries to this directory
      for ( int i = 0; i < lstFiles.length; i++ )
      {
        // The current entry
        File oEntry = lstFiles[ i ];

        if ( oEntry.isFile() )
        {
          // Flag, whether the entry matches the search criteria
          boolean bMatches = false;

          // Check the entry's name for a match
          if ( oEntry.getName().indexOf( strNamePattern ) >= 0 )
          {
            // Check the entry's contents for a match
            if ( SearchFile( oEntry, strContentPattern ) )
            {
              // Found a content match
              bMatches = true;
            }
          }
          else
          {
            // No match is found
            bMatches = false;
          }

          if ( bMatches )
            lst.add( oEntry.getAbsolutePath() );
        }
        else if ( oEntry.isDirectory() )
        {
          String[] lstFound = SearchDirectory( oEntry.getAbsolutePath(), strNamePattern, strContentPattern );

          if ( null != lstFound )
          {
            for ( int j = 0; j < lstFound.length; j++ )
              lst.add( (String)lstFound[ j ] );
          }
        }
      }
    }

    String[] lstToReturn = new String[ lst.size() ];
    Iterator it = lst.iterator();

    for ( int i = 0; i < lstToReturn.length; i++ )
    {
      lstToReturn[ i ] = (String)it.next();
    }

    return lstToReturn;
  }

  /**
   * Search the contents of a specified file.
   *
   * Note:  If the strPattern parameter is null, then the routine returns
   *        false without opening the file at all.
   */
  private boolean SearchFile( File oFile, String strPattern )
  {
    if ( null == strPattern )
      return true;

    if ( null == oFile )
      return false;

    // Search the file's contents

    // Pessimism
    boolean bMatches = false;

    if ( oFile.exists() && oFile.isFile() && oFile.canRead() && oFile.length() >= strPattern.length() )
    {
      try
      {
        // Open the file
        FileInputStream is = new FileInputStream( oFile );
        BufferedInputStream fis = new BufferedInputStream( is );

        // Get the pattern's bytes
        byte[] bytes = strPattern.getBytes();
        byte[] buf = new byte[ strPattern.length() ];

        int iStreamPos = 0;

        fis.mark( (int)oFile.length() );

        while ( fis.read( buf ) == buf.length )
        {
          for ( int i = 0; i < buf.length; i++ )
          {
            if ( buf[ i ] != bytes[ i ] )
            {
              bMatches = false;
              break;
            }
            else
              bMatches = true;
          }

          if ( bMatches )
            break;

          fis.reset();
          fis.skip( ++iStreamPos );
        }

        // Close the file
        fis.close();
        fis = null;
      }
      catch ( FileNotFoundException e )
      {
        // TODO: May be redirect to the error page?
        e.printStackTrace();
      }
      catch ( IOException e )
      {
        // TODO: May be redirect to the error page?
        e.printStackTrace();
      }
    }

    // Return the value
    return bMatches;
  }
}