package jfilesearch;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

/**
 * Title:        Java File Search and Download Engine
 * Description:  A simple java servlets and JSPs implementation of a search engine.
 * Copyright:    Copyright (c) 2002
 * Company:      FMI
 * @author Kaloian Manassiev
 * @version 1.0
 */

public class CFileSearchServlet extends HttpServlet
{
  // Static constants
  private static final String CONTENT_TYPE = "text/html";
  public  static final String SEARCH_ROOT_INIT_PARAMETER = "SEARCH_ROOT";
  private static final String SEARCH_PATTERN_PARAMETER = "SearchPattern";
  private static final String SEARCH_WITHIN_CONTENTS_PARAMETER = "SearchContents";
  private static final String BOOLEAN_TRUE = "TRUE";

  // Private data
  private String[]  m_lstMatchedItems;

  // The pattern to be matched agains the files.
  private String  m_strSearchPattern;

  // The search root
  private String  m_strSearchRoot;

  // Whether the search should be performed within file contents
  private boolean m_bSearchContents;

  /////////////////////// Private business methods //////////////////////////////////

  /**
   *  Trims the leading drive specification (if such exists)
   */
  private String TrimDriveLetter( String strPath )
  {
    if ( null == strPath || strPath.length() == 0 )
      return "";

    // Trim the string of leading and trailing whitespace
    strPath.trim();

    int iPos = strPath.indexOf( ":" );

    if ( iPos > 0 )
    {
      strPath = strPath.substring( iPos + 1 );
    }

    return strPath;
  }

  /**
   * Writes the header contents
   * */
  private void WriteHeader( PrintWriter out )
  {
    out.write(  "<HEAD>" +
                "<TITLE>Search engine servlet results" +
                ( m_bSearchContents ? " (Searching within contents)" : "" ) +
                "</TITLE>" +
                "</HEAD>");
  }

  /**
   * Writes the search results
   * */
  private void WriteBody( PrintWriter out )
  {
    out.write( "<BODY>\n<P>\n" );

    if ( null == m_lstMatchedItems )
    {
      out.write( "No items were found matching the search criteria of&nbsp&nbsp[ <b>" + m_strSearchPattern + "</b> ]" );
    }
    else
    {
      out.write(  m_lstMatchedItems.length + " items matching your search criteria of [<b> " +
                  m_strSearchPattern + "</b> ] were found at " + m_strSearchRoot + ".<br>" +
                  "Search within the contents was" + ( m_bSearchContents ? " " : " NOT " ) +
                  "performed.<br><br>" );

      for ( int i = 0; i < m_lstMatchedItems.length; i++ )
      {
        File f = new File( m_lstMatchedItems[ i ] );
        String strCanonical = "";

        try
        {
          strCanonical = TrimDriveLetter( f.getCanonicalPath() );
        }
        catch ( IOException e )
        {
          strCanonical = "";
        }

        out.write(  "<A href=\"FileDownload?" +
                    CFileDownloadServlet.DOWNLOAD_FILE_PARAMETER + "=" + m_lstMatchedItems[ i ] +
                    "\">" + "(" + i + ") " + strCanonical + "</A><br>" );
      }
    }

    out.write( "\n</P>\n</BODY>" );
  }


  /////////////////////// HttpServlet methods implementation ///////////////////////

  /**
   * Initialization of the global variables
   * */
  public void init( ServletConfig config ) throws ServletException
  {
    // Let the parent perform its initialization
    super.init( config );

    // Release the list with matched items
    m_lstMatchedItems = null;
    m_strSearchPattern = null;
  }

  /**
   * Process the HTTP Get request
   * */
  public void doGet( HttpServletRequest request, HttpServletResponse response )
    throws ServletException, IOException
  {
    try
    {
      // Read environment parameters
      m_strSearchRoot = getInitParameter( SEARCH_ROOT_INIT_PARAMETER );
      m_strSearchPattern = request.getParameter( SEARCH_PATTERN_PARAMETER );

      try
      {
        m_bSearchContents = ( request.getParameter( SEARCH_WITHIN_CONTENTS_PARAMETER ).equalsIgnoreCase( BOOLEAN_TRUE ) ? true : false );
      }
      catch ( Exception e )
      {
        m_bSearchContents = false;
      }

      // Perform the search
      CFileSearch fs = new CFileSearch( m_strSearchRoot );

      m_lstMatchedItems = fs.DoSearch( m_strSearchPattern, ( m_bSearchContents ? m_strSearchPattern : null ) );
    }
    catch( Exception e )
    {
      e.printStackTrace();

      // Clear the found items list
      m_lstMatchedItems = null;
    }

    // Set the HTML content type
    response.setContentType( CONTENT_TYPE );

    // Obtain the output stream
    PrintWriter out = response.getWriter();

    // The HTML start
    out.println("<HTML>\n");

    // Write the header contents
    WriteHeader( out );

    // Write the search results
    WriteBody( out );

    // The HTML end
    out.println("\n</HTML>");
  }

  /**
   * Process the HTTP Post request
   * */
  public void doPost( HttpServletRequest request, HttpServletResponse response )
    throws ServletException, IOException
  {
    // Do the same as the Get request does
    doGet( request, response );
  }

  /**
   * Clean up resources
   * */
  public void destroy()
  {
  }
}